#include "int_lists.h"

/* Allocate memory, exit with an error-message, if no memory 
   available */ 

void* SecureMalloc(size_t size)
{
   void* res = malloc(size);

   if(!res)
   {
      printf("malloc() failure -- out of memory?");
      exit(EXIT_FAILURE);
   }
   return res;
 }


/* Insert a new integer as the first element of an integer list */

void IntListInsertFirst(IntList_p *list, int new_val)
{
   IntList_p handle;

   handle = SecureMalloc(sizeof (IntListCell));
   handle->value = new_val;
   handle->next = *list;
   *list = handle;
}

/* Helper function: Given a non-empty list, return last element */

IntList_p find_last_element(IntList_p list)
{
   if(list->next)
   {
      return find_last_element(list->next);
   }
   return list;
}

/* Insert a new integer as the last element of an integer list */

void IntListInsertLast(IntList_p *list, int new_val)
{
   IntList_p handle, last;

   handle = SecureMalloc(sizeof (IntListCell));
   handle->value = new_val;
   handle->next = NULL;  

   if(!*list)
   {
      *list = handle;
   }
   else
   {
      last = find_last_element(*list);
      last->next = handle;
   }
}

/* Free the memory taken up by a list */

void IntListFree(IntList_p list)
{
   if(list)
   {
      IntListFree(list->next); /* Free rest */
      free(list); /* Free this cell */
   }
}

/* Print a list as a sequence of numbers */   

void IntListPrint(IntList_p list)
{
   IntList_p handle;

   for(handle = list; handle; handle = handle->next)
   {
      printf("%d ", handle->value);
   }
   putchar('\n');
}


